/*
 * Copyright (C)2010-2012 Henning Karlsen. All right reserved.
 * Copyright (C)2012 Todor Todorov.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to:
 *
 * Free Software Foundation, Inc.
 * 51 Franklin St, 5th Floor, Boston, MA 02110-1301, USA
 *
 *********************************************************************/
#include "ssd1289.h"
#include "helpers.h"

SSD1289_LCD::SSD1289_LCD( PinName CS, PinName RESET, PinName RS, PinName WR, BusOut* DATA_PORT, PinName BL, PinName RD )
    : LCD( 240, 320, CS, RS, RESET ), _lcd_pin_wr( WR )
{
    _lcd_port = DATA_PORT;
    if ( BL != NC ) _lcd_pin_bl = new DigitalOut( BL );
    else _lcd_pin_bl = 0;
    if ( RD != NC ) _lcd_pin_rd = new DigitalOut( RD );
    else _lcd_pin_rd = 0;
}

void SSD1289_LCD::Initialize( orientation_t orientation, colordepth_t colors )
{
    _orientation = orientation;
    _colorDepth = RGB16;
    
    _lcd_pin_reset = HIGH;
    wait_ms( 5 );
    _lcd_pin_reset = LOW;
    wait_ms( 15 );
    _lcd_pin_reset = HIGH;
    _lcd_pin_cs = HIGH;
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = HIGH;
    if ( _lcd_pin_rd != 0 )
        *_lcd_pin_rd = HIGH;
    _lcd_pin_wr = HIGH;
    wait_ms( 15 );
    
    Activate();
    WriteCmdData( 0x00, 0x0001 ); wait_ms( 1 );
    WriteCmdData( 0x03, 0xA8A4 ); wait_ms( 1 );
    WriteCmdData( 0x0C, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x0D, 0x080C ); wait_ms( 1 );
    WriteCmdData( 0x0E, 0x2B00 ); wait_ms( 1 );
    WriteCmdData( 0x1E, 0x00B7 ); wait_ms( 1 );
    WriteCmdData( 0x01, 0x2B3F ); wait_ms( 1 );
    WriteCmdData( 0x02, 0x0600 ); wait_ms( 1 );
    WriteCmdData( 0x10, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x11, 0x6070 ); wait_ms( 1 );
    WriteCmdData( 0x05, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x06, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x16, 0xEF1C ); wait_ms( 1 );
    WriteCmdData( 0x17, 0x0003 ); wait_ms( 1 );
    WriteCmdData( 0x07, 0x0233 ); wait_ms( 1 );
    WriteCmdData( 0x0B, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x0F, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x41, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x42, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x48, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x49, 0x013F ); wait_ms( 1 );
    WriteCmdData( 0x4A, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x4B, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x44, 0xEF00 ); wait_ms( 1 );
    WriteCmdData( 0x45, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x46, 0x013F ); wait_ms( 1 );
    WriteCmdData( 0x30, 0x0707 ); wait_ms( 1 );
    WriteCmdData( 0x31, 0x0204 ); wait_ms( 1 );
    WriteCmdData( 0x32, 0x0204 ); wait_ms( 1 );
    WriteCmdData( 0x33, 0x0502 ); wait_ms( 1 );
    WriteCmdData( 0x34, 0x0507 ); wait_ms( 1 );
    WriteCmdData( 0x35, 0x0204 ); wait_ms( 1 );
    WriteCmdData( 0x36, 0x0204 ); wait_ms( 1 );
    WriteCmdData( 0x37, 0x0502 ); wait_ms( 1 );
    WriteCmdData( 0x3A, 0x0302 ); wait_ms( 1 );
    WriteCmdData( 0x3B, 0x0302 ); wait_ms( 1 );
    WriteCmdData( 0x23, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x24, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x25, 0x8000 ); wait_ms( 1 );
    WriteCmdData( 0x4f, 0x0000 ); wait_ms( 1 );
    WriteCmdData( 0x4e, 0x0000 ); wait_ms( 1 );
    WriteCmd( 0x22 );
    Deactivate();
}

void SSD1289_LCD::Sleep( void )
{
    // TODO: figure out if the SSD1289 controller has sleep and wakeup commands
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = LOW;
}

void SSD1289_LCD::WakeUp( void )
{
    // TODO: figure out if the SSD1289 controller has sleep and wakeup commands
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = HIGH;
}

void SSD1289_LCD::WriteCmd( unsigned short cmd )
{
    _lcd_pin_rs = LOW;
    _lcd_port->write( cmd );
    pulseLow( _lcd_pin_wr );
}

void SSD1289_LCD::WriteData( unsigned short data )
{
    _lcd_pin_rs = HIGH;
    _lcd_port->write( data );
    pulseLow( _lcd_pin_wr );
}

void SSD1289_LCD::SetXY( uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2 )
{
    if ( _orientation == LANDSCAPE )
    {
        swap( uint16_t, x1, y1 )
        swap( uint16_t, x2, y2 )
        y1 = _disp_height - y1;
        y2 = _disp_height - y2;
        swap( uint16_t, y1, y2 )
    }

    WriteCmdData( 0x44, ( x2 << 8 ) + x1 );
    WriteCmdData( 0x45, y1 );
    WriteCmdData( 0x46, y2 );
    WriteCmdData( 0x4e, x1 );
    WriteCmdData( 0x4f, y1 );
    WriteCmd( 0x22 );
}

void SSD1289_LCD::SetPixelColor( unsigned int color )
{
    unsigned char r, g, b;
    r = ( color >> 16 ) & 0xFF;
    g = ( color >> 8 ) & 0xFF;
    b = color & 0xFF;
    unsigned short clr = ( ( ( ( r ) & 0xF8 ) | ( ( g ) >> 5 ) ) << 8 ) | ( ( ( ( g ) & 0x1C ) << 3 ) | ( ( b ) >> 3 ) );
    WriteData( clr );
}
